using System;
using System.Collections.Generic;
using System.Linq;
using MacGruber;
using SimpleJSON;
using UnityEngine;

namespace CheesyFX
{
    public class TouchTrigger : MonoBehaviour
    {
        public new string name;
        private TouchZone region;
        // private JSONStorableFloat driver;
        private bool absolute;
        private EventTrigger enterTrigger;
        private EventTrigger exitTrigger;
        private EventTrigger timeExceededTrigger;
        private FloatTrigger timeTrigger;
        private JSONStorableFloat inputTo;
        private JSONStorableFloat inputFrom;
        private JSONStorableFloat threshold;
        public JSONStorableFloat resetRate;
        public JSONStorableBool instantReset;
        private bool panelOpen;
        private JSONStorableString info = new JSONStorableString("info", "");
        private string baseInfo;
        private float lastValue;

        private bool hasRange = true;
        
        List<object> UIElements = new List<object>();
        // public Condition condition;

        public void Init(TouchZone region, float thresholdDefault = 1f, float fromDefault = 0f, float toDefault = 5f)
        {
            // this.driver = driver;
            // var regionSplit = region.Split(new[] { "_" }, StringSplitOptions.None);
            // name = driver.name+"Trigger"+regionSplit[1];
            // baseInfo = $"{regionSplit[0]} {this.driver.name}Trigger";
            // baseInfo = $"{regionSplit[0]} {this.driver.name}Trigger{regionSplit[1]}";
            name = $"{region.name}";
            baseInfo = $"{region.name}: Time Touched";
            this.region = region;
            region.touchCollisionListener.touchTrigger = this;
            inputFrom = new JSONStorableFloat("Input From", fromDefault, val => hasRange = val != inputTo.val, 0f, 1f, false);
            inputTo = new JSONStorableFloat("Input To", toDefault, val => hasRange = val != inputFrom.val, 0f, 1f, false);
            threshold = new JSONStorableFloat("Threshold", thresholdDefault, 0f, 1f, false);
            resetRate = new JSONStorableFloat("Reset Rate", 10f, 0f, 20f, false);
            instantReset = new JSONStorableBool("Instant Reset", false, val => resetRate.SetInteractble(!val));
            enterTrigger = new EventTrigger(TouchMe.singleton, "OnEnter");
            exitTrigger = new EventTrigger(TouchMe.singleton, "OnExit");
            timeExceededTrigger = new EventTrigger(TouchMe.singleton, "OnThresholdExceeded");
            timeTrigger = new FloatTrigger(TouchMe.singleton, "OnValueChanged");

            // condition = new Condition();

            resetRate.setCallbackFunction += OnResetRateChanged;

            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            SuperController.singleton.onAtomAddedHandlers += OnAtomAdded;
        }

        public void Trigger(float v)
        {
            if(timeTrigger.HasActions())
            {
                if (hasRange)
                {
                    timeTrigger.Trigger((v - inputFrom.val) / (inputTo.val - inputFrom.val));

                }
                else timeTrigger.Trigger(v <= inputFrom.val? 0f : 1f);
                timeTrigger.Update();
            }
            if(timeExceededTrigger.HasActions())
            {
                // if (lastValue < threshold.val && v >= threshold.val) timeExceededTrigger.Trigger();
                if (lastValue < threshold.val)
                {
                    if (v >= threshold.val) timeExceededTrigger.Trigger();
                }
                else if (v < threshold.val) exitTrigger.Trigger();
                lastValue = v;
            }
        }

        private void Update()
        {
            enterTrigger.Update();
            exitTrigger.Update();
            
            // if (condition != null && !condition.IsMet()) return;
            if (panelOpen) info.val = $"{baseInfo}\n{region.timeTouched:0.00}";
            Trigger(region.timeTouched);
            if (Math.Abs(lastValue - region.timeTouched) > .001f)
            {
                timeTrigger.Update();
                // region.name.Print();
            }
        }

        public void OnDestroy()
        {
            enterTrigger.Remove();
            exitTrigger.Remove();
            timeTrigger.Remove();
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            SuperController.singleton.onAtomAddedHandlers -= OnAtomAdded;
        }

        public void OnAtomRename(string oldUid, string newUid)
        {
            enterTrigger.SyncAtomNames();
            exitTrigger.SyncAtomNames();
            timeTrigger.SyncAtomNames();
            // condition.OnAtomRename();
        }

        public void OnAtomAdded(Atom atom)
        {
            // condition.AddExclusionChoice(atom);
        }

        private void OnResetRateChanged(float val)
        {
            if(val > 0f)
            {
                if (!region.touchCollisionListener.isOnStay && region.touchCollisionListener.touchTimerReset == null)
                {
                    region.touchCollisionListener.touchTimerReset =
                        region.touchCollisionListener.TouchTimerReset().Start();
                }
            }
            else if(!region.touchCollisionListener.isOnStay)
            {
                region.touchCollisionListener.touchTimerReset.Stop();
            }
        }

        public JSONClass Store(string subScenePrefix)
        {
            JSONClass jc = new JSONClass();
            inputFrom.Store(jc);
            inputTo.Store(jc);
            threshold.Store(jc);
            jc[enterTrigger.Name] = enterTrigger.GetJSON(subScenePrefix);
            jc[exitTrigger.Name] = exitTrigger.GetJSON(subScenePrefix);
            jc[timeTrigger.Name] = timeTrigger.GetJSON(subScenePrefix);
            // condition.excludedInfo.Store(jc);
            return jc;
        }

        public void Load(JSONClass jc, string subScenePrefix)
        {
            inputFrom.Load(jc);
            inputTo.Load(jc);
            threshold.Load(jc);
            // condition.excludedInfo.Load(jc);
            enterTrigger.RestoreFromJSON(jc, subScenePrefix, false, true);
            exitTrigger.RestoreFromJSON(jc, subScenePrefix, false, true);
            timeTrigger.RestoreFromJSON(jc, subScenePrefix, false, true);
        }

        public void OpenPanel(MVRScript script, Action back)
        {
            panelOpen = true;
            
            UIDynamicButton button;
            button = script.CreateButton("Return");
            button.buttonColor = new Color(0.55f, 0.90f, 1f);
            button.button.onClick.AddListener(
                () =>
                {
                    Utils.RemoveUIElements(script, UIElements);
                    panelOpen = false;
                    back();
                });
            UIElements.Add(button);
            
            resetRate.CreateUI(UIElements: UIElements, rightSide:false);
            
            // Utils.SetupInfoText(script, name, 80f, true);
            var textfield = script.CreateTextField(info, true);
            textfield.ForceHeight(120f);
            textfield.UItext.fontSize = 36;
            textfield.UItext.alignment = TextAnchor.MiddleCenter;
            UIElements.Add(textfield);
            
            instantReset.CreateUI(UIElements: UIElements,rightSide:true);
            
            var spacer = script.CreateSpacer(true);
            spacer.height = 30f;
            UIElements.Add(spacer);
            
            UIManager.SetupInfoOneLine(UIElements,"Threshold Triggers", false, script: script);
            // var spacer = script.CreateSpacer(true);
            // spacer.height = 50f;
            // triggerUIElements.Add(spacer);
            // UIManager.EvenLeftToRight(script, triggerUIElements);

            threshold.CreateUI(script, true, UIElements:UIElements);
            button = script.CreateButton("On Threshold Exceeded", false);
            button.button.onClick.AddListener(timeExceededTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            UIElements.Add(button);
            
            button = script.CreateButton("On Threshold Undershot", false);
            button.button.onClick.AddListener(exitTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            UIElements.Add(button);

            UIManager.SetupInfoOneLine(UIElements,"Value Triggers", false, script: script);
            spacer = script.CreateSpacer(true);
            spacer.height = 30f;
            UIElements.Add(spacer);
            // UIManager.EvenLeftToRight(script, triggerUIElements);
            
            inputFrom.CreateUI(script, UIElements:UIElements);
            inputTo.CreateUI(script, true, UIElements:UIElements);
            
            
            button = script.CreateButton("On Value Changed", false);
            button.button.onClick.AddListener(timeTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            UIElements.Add(button);
            
            

            // if (condition == null) return;
            // UIManager.SetupInfoOneLine(triggerUIElements,"Conditions", true, script: script);
            // // UIManager.EvenLeftToRight();
            // button = script.CreateButton("Exclude...");
            // button.button.onClick.AddListener(() => condition.Exclude(condition.excludedChooser.val, true));
            // triggerUIElements.Add(button);
            // button = script.CreateButton("(Re)Include...");
            // button.button.onClick.AddListener(() => condition.Exclude(condition.excludedChooser.val, false));
            // triggerUIElements.Add(button);
            // condition.excludedChooser.CreateUI(script: script, rightSide: true,chooserType:2, UIElements:triggerUIElements);
            //
            // textfield = script.CreateTextField(condition.excludedInfo, true);
            // textfield.height = 200f;
            // triggerUIElements.Add(textfield);
        }
    }

    public class Condition
    {
        public JSONStorableStringChooser excludedChooser =
            new JSONStorableStringChooser("atomChooser", null, "", "... this Atom");
        public static HashSet<Atom> excludedAtoms = new HashSet<Atom>();
        public JSONStorableString excludedInfo = new JSONStorableString("excludeInfo", "<b>Excluded Atoms</b>\n");

        public Atom atom;

        public Condition()
        {
            excludedChooser.choices = SuperController.singleton.GetAtomUIDs();
        }

        public bool IsMet()
        {
            return !excludedAtoms.Contains(atom);
        }

        public void Exclude(string uid, bool val)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(uid);
            if (atom == null) return;
            if (val)
            {
                if(excludedAtoms.Add(atom))
                {
                    excludedInfo.val += atom.uid + "\n";
                }
            }
            else if(excludedAtoms.Remove(atom))
            { 
                excludedInfo.val = "<b>Excluded Atoms</b>\n"+string.Join("\n", excludedAtoms.Select(x => x.uid).ToArray());
            }
        }
        
        // public void ResetExcluded()
        // {
        //     excludedAtoms.Clear();
        //     excludedInfo.val = "<b>Excluded Atoms</b>\n";
        // }
        //
        // public void AddExclusionChoice(Atom atom)
        // {
        //     if (excludedChooser.choices.Contains(atom.uid)) return;
        //     var choices = excludedChooser.choices;
        //     choices.Insert(0, atom.uid);
        //     excludedChooser.choices = new List<string>(choices);
        // }
        //
        // public void OnAtomRename()
        // {
        //     excludedChooser.choices = SuperController.singleton.GetAtomUIDs();
        //     var ignoredUids = excludedAtoms.Select(x => x.uid).ToArray();
        //     excludedInfo.val = "<b>Excluded Atoms</b>\n"+string.Join("\n", ignoredUids);
        // }
    }
}