using System;
using System.Collections.Generic;
using System.Linq;
using PrefabEvolution;
using SimpleJSON;
using UnityEngine;
using Object = UnityEngine.Object;

namespace CheesyFX
{
    public class TouchTriggers
    {
        private static List<object> UIElements = new List<object>();
        private static UIDynamicButton addButton;
        private static UIDynamicButton removeButton;

        private static string selectedRegion;

        private static JSONStorableStringChooser regionChooser =
            new JSONStorableStringChooser("regionChooser", BodyRegionMapping.touchZoneNames, "", "Region", OnRegionSelected);

        private static JSONStorableBool showRegionsWithTrigger = new JSONStorableBool("Show Only Regions With Trigger", false, OnShowRegionsChanged);

        public static List<object> CreateUI()
        {
            showRegionsWithTrigger.CreateUI(UIElements: UIElements, rightSide: false);
            regionChooser.CreateUI(UIElements:UIElements, chooserType:2);
            if (String.IsNullOrEmpty(selectedRegion))
            {
                var regionWithTrigger = BodyRegionMapping.touchZones.Values.FirstOrDefault(x => x.touchCollisionListener.touchTrigger != null);
                if (regionWithTrigger != null)
                {
                    CreateButtons(regionWithTrigger.touchCollisionListener.touchTrigger);
                    regionChooser.valNoCallback = regionWithTrigger.name;
                }
            }
            else CreateButtons(BodyRegionMapping.touchZones[selectedRegion].touchCollisionListener.touchTrigger);
            return UIElements;
        }

        private static void CreateButtons(TouchTrigger touchTrigger)
        {
            TouchMe.singleton.RemoveButton(addButton);
            TouchMe.singleton.RemoveButton(removeButton);
            if (touchTrigger == null)
            {
                addButton = TouchMe.singleton.CreateButton("Add Touch Trigger", true);
                addButton.button.onClick.AddListener(() =>
                {
                    {
                        TouchMe.singleton.gameObject.AddComponent<TouchTrigger>().Init(BodyRegionMapping.touchZones[regionChooser.val]);
                        CreateUI();
                    }
                });
            }
            else
            {
                addButton = TouchMe.singleton.CreateButton("Open Trigger Panel", true);
                addButton.button.onClick.AddListener(() =>
                {
                    {
                        TouchMe.singleton.ClearUI();
                        touchTrigger.OpenPanel(TouchMe.singleton, () => TouchMe.singleton.CreateUI());
                    }
                });
                removeButton = TouchMe.singleton.CreateButton("Remove Trigger", true);
                removeButton.button.onClick.AddListener(() =>
                {
                    Object.Destroy(touchTrigger);
                    touchTrigger = null;
                    CreateButtons(touchTrigger);
                });
                UIElements.Add(removeButton);
            }
            addButton.buttonColor = new Color(0.45f, 1f, 0.45f);
            UIElements.Add(addButton);
        }

        private static void OnRegionSelected(string name)
        {
            selectedRegion = name;
            CreateUI();
        }

        private static void OnShowRegionsChanged(bool val)
        {
            if (val)
            {
                regionChooser.choices = BodyRegionMapping.touchZones.Values.Where(x => x.touchCollisionListener.touchTrigger != null).Select(x => x.name).ToList();
                if (!regionChooser.choices.Contains(regionChooser.val))
                {
                    if (regionChooser.choices.Count == 0)
                    {
                        regionChooser.valNoCallback = string.Empty;
                        TouchMe.singleton.RemoveButton(addButton);
                        TouchMe.singleton.RemoveButton(removeButton);
                    }
                    else regionChooser.val = regionChooser.choices[0];
                }
            }
            else
            {
                regionChooser.choices = BodyRegionMapping.touchZoneNames;
                regionChooser.val = selectedRegion;
            }
        }

        public static JSONClass Store()
        {
            JSONClass jc = new JSONClass();
            foreach (var tz in BodyRegionMapping.touchZones.Values.ToList())
            {
                if (tz.touchCollisionListener.touchTrigger != null) jc[tz.name] = tz.touchCollisionListener.touchTrigger.Store(TouchMe.singleton.subScenePrefix);
            }
            return jc;
        }
        
        public static void Load(JSONClass jc)
        {
            if(jc.HasKey("TouchTriggers"))
            {
                foreach (var name in jc["TouchTriggers"].AsObject.Keys)
                {
                    TouchMe.singleton.gameObject.AddComponent<TouchTrigger>().Init(BodyRegionMapping.touchZones[name]);
                    BodyRegionMapping.touchZones[name].touchCollisionListener.touchTrigger.Load(jc["TouchTriggers"][name].AsObject, TouchMe.singleton.subScenePrefix);
                }
            }
        }
    }
}